import { SLine, SPoint, SRect } from "@persagy-web/draw";

export class SMathUtil {
    /**
     * 计算点到点距离
     *
     * @return  距离
     * @param x1
     * @param y1
     * @param x2
     * @param y2
     */
    static pointDistance(
        x1: number,
        y1: number,
        x2: number,
        y2: number
    ): number {
        return Math.sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2));
    } // Function pointDistance()

    /**
     * 计算矩形是否有交集(外包矩形算法)
     *
     * @param   rect1   矩形1
     * @param   rect2   矩形2
     * @return  boolean
     * */
    static rectIntersection(rect1: SRect, rect2: SRect): boolean {
        let minX = rect1.x < rect2.x ? rect1.x : rect2.x;
        let minY = rect1.y < rect2.y ? rect1.y : rect2.y;
        let maxX = rect1.right > rect2.right ? rect1.right : rect2.right;
        let maxY = rect1.bottom > rect2.bottom ? rect2.bottom : rect2.bottom;
        return (
            rect1.width + rect2.width > maxX - minX &&
            rect1.height + rect2.height > maxY - minY
        );
    } // Function rectIntersection()

    /**
     * 计算线段交点
     *
     * @param   line1   线段1
     * @param   line2   线段2
     * @return  SPoint  交点 null 平行但不重合 'repeat' 重合
     */
    static lineIntersection(
        line1: SLine,
        line2: SLine
    ): SPoint | null | string {
        let k1 = line1.dy / line1.dx;
        let b1 = line1.y1 - k1 * line1.x1;
        let k2 = line2.dy / line2.dx;
        let b2 = line2.y1 - k2 * line2.x1;
        if (k1 == k2) {
            if (b1 == b2) {
                return "repeat";
            }
            return null;
        }
        let intersectionX = (b2 - b1) / (k1 - k2);
        let intersectionY = k1 * intersectionX + b1;
        let minX = Math.min(line1.x1, line1.x2);
        let maxX = Math.min(line2.x1, line2.x2);
        if (intersectionX >= minX && intersectionX <= maxX) {
            return new SPoint(intersectionX, intersectionY);
        }
        return null;
    } // Function lineIntersection()

    /**
     * 转化数据格式为[][]
     *
     * @param  SP  SPoint[]
     * @return Arr number[][]
     */
    static transferToArray(SP: SPoint[]): number[][] {
        return SP.map((t): number[] => {
            return [t.x, t.y];
        });
    } // Function transferToArray()

    /**
     * 转化数据格式为SPoint[]
     *
     * @param   arr number[][]
     * @return  SP  SPoint[]
     */
    static transferToSPoint(arr: number[][]): SPoint[] {
        return arr.map(
            (t): SPoint => {
                return new SPoint(t[0], t[1]);
            }
        );
    } // Function transferToSPoint()

    /**
     * 计算轮廓线面积
     * */
    static calculateArea(arr: SPoint[]): number {
        let sum = 0;
        let n = arr.length;
        arr[n] = arr[0];
        for (let i = 1; i <= n; i++) {
            sum += arr[i].x * arr[i - 1].y - arr[i - 1].x * arr[i].y;
        }
        return sum / 2;
    } // Function calculateArea()
} // Class SMathUtil
